with AUnit.Assertions; 
with Crypto.Symmetric.Blockcipher_Twofish128;
with Crypto.Types;

package body Test.Twofish128 is
use Crypto.Types;
   
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
-------------------------------- Type - Declaration --------------------------------
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

   Plaintext: B_Block128 := (others => 0);
   Key: B_Block128 := (others => 0);
   Ciphertext: B_Block128 := (16#5D#, 16#9D#, 16#4E#, 16#EF#, 16#FA#,
                              16#91#, 16#51#, 16#57#, 16#55#, 16#24#,
                              16#F1#, 16#15#, 16#81#, 16#5A#, 16#12#,
                              16#E0#);
   PT: B_Block128 := Plaintext;
   CT: B_Block128;

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
---------------------------- Register Twofish128 Test 1 ----------------------------
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
	
	procedure Register_Tests(T : in out Twofish_Test) is
		use Test_Cases.Registration;
	begin
		Register_Routine(T, Twofish128_Test1'Access,"Twofish128_Test1.");
		Register_Routine(T, Twofish128_Test2'Access,"Twofish128_Test2.");
		Register_Routine(T, Twofish128_Test3'Access,"Twofish128_Test3.");
	end Register_Tests;

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
------------------------------ Name Twofish128 Test ------------------------------
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

	function Name(T : Twofish_Test) return Test_String is
	begin
		return new String'("Twofish128 Test");
	end Name;

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
------------------------------------ Start Tests -----------------------------------
------------------------------------------------------------------------------------
-------------------------------------- Test 1 --------------------------------------
------------------------------------------------------------------------------------

   procedure Twofish128_Test1(T : in out Test_Cases.Test_Case'Class) is
      use AUnit.Assertions; 
      use Crypto.Symmetric.Blockcipher_Twofish128;
   begin
   	   for I in 1 .. 49 loop
   	   	   Prepare_Key(Key);
   	   	   Encrypt(PT, CT);
   	   	   Key := PT;
   	   	   PT  := CT;
   	   end loop;
   	   
   	   for I in CT'Range loop
   	   	   Assert(CT(I) = Ciphertext(I), "Twofish128 failed.");
	   end loop;
   end Twofish128_Test1;

------------------------------------------------------------------------------------
-------------------------------------- Test 2 --------------------------------------
------------------------------------------------------------------------------------

   procedure Twofish128_Test2(T : in out Test_Cases.Test_Case'Class) is
      use AUnit.Assertions; 
      use Crypto.Symmetric.Blockcipher_Twofish128;
   begin
   	   
   	   Key := (16#BC#, 16#A7#, 16#24#, 16#A5#, 16#45#, 16#33#, 16#C6#, 16#98#,
               16#7E#, 16#14#, 16#AA#, 16#82#, 16#79#, 16#52#, 16#F9#, 16#21#);
       
       CT := (16#6B#, 16#45#, 16#92#, 16#86#, 16#F3#, 16#FF#, 16#D2#, 16#8D#,
              16#49#, 16#F1#, 16#5B#, 16#15#, 16#81#, 16#B0#, 16#8E#, 16#42#);
       
       Prepare_Key(Key);
       Decrypt(Ciphertext, PT);
   	   
   	   for I in PT'Range loop
   	   	   Assert(PT(I) = CT(I), "Twofish128 failed.");
	   end loop;

   end Twofish128_Test2;

------------------------------------------------------------------------------------
-------------------------------------- Test 3 --------------------------------------
------------------------------------------------------------------------------------

   procedure Twofish128_Test3(T : in out Test_Cases.Test_Case'Class) is
      use AUnit.Assertions; 
      use Crypto.Symmetric.Blockcipher_Twofish128;
   begin
   	   
   	   Key := (others => 0);
   	   
   	   CT := (16#9F#, 16#58#, 16#9F#, 16#5C#, 16#F6#, 16#12#, 16#2C#, 16#32#,
              16#B6#, 16#BF#, 16#EC#, 16#2F#, 16#2A#, 16#E8#, 16#C3#, 16#5A#);
       
       Prepare_Key(Key);
       Decrypt(CT, PT);
   	   
	   CT := (others => 0); 
   	   for I in PT'Range loop
   	   	   Assert(PT(I) = CT(I), "Twofish128 failed.");
	   end loop;

   end Twofish128_Test3;

------------------------------------------------------------------------------------

end Test.Twofish128;
