with AUnit.Assertions;
with Crypto.Symmetric.Oneway_Blockcipher_Twofish192;
with Crypto.Types;

package body Test.Twofish192_Oneway is
   use Crypto.Types;

   -----------------------------------------------------------------------------
   -----------------------------------------------------------------------------
   -------------------------- Type - Declaration -------------------------------
   -----------------------------------------------------------------------------
   -----------------------------------------------------------------------------

   Plaintext : B_Block128 := (others => 0);
   Key: Bytes(B_Block192'Range) := (others => 0);
   Ciphertext: B_Block128 := (16#E7#, 16#54#, 16#49#, 16#21#, 16#2B#,
                              16#EE#, 16#F9#, 16#F4#, 16#A3#, 16#90#,
                              16#BD#, 16#86#, 16#0A#, 16#64#, 16#09#,
                              16#41#);
   PT: B_Block128 := Plaintext;
   CT: B_Block128;

   -----------------------------------------------------------------------------
   -----------------------------------------------------------------------------
   -------------------------- Register Twofish192 Test 1 -----------------------
   -----------------------------------------------------------------------------
   -----------------------------------------------------------------------------

   procedure Register_Tests(T : in out Twofish_Test) is
      use Test_Cases.Registration;
   begin
      Register_Routine(T, Twofish192_Oneway_Test1'Access,"Twofish192 Known Answer Test");

   end Register_Tests;

   -----------------------------------------------------------------------------
   -----------------------------------------------------------------------------
   ------------------------- Name Twofish192 Test ------------------------------
   -----------------------------------------------------------------------------
   -----------------------------------------------------------------------------

   function Name(T : Twofish_Test) return Test_String is
   begin
      return new String'("Twofish192 Test");
   end Name;

   -----------------------------------------------------------------------------
   -----------------------------------------------------------------------------
   -------------------------------- Start Tests --------------------------------
   -----------------------------------------------------------------------------
   ----------------------------------- Test 1 ----------------------------------
   -----------------------------------------------------------------------------

   procedure Twofish192_Oneway_Test1(T : in out Test_Cases.Test_Case'Class) is
      use AUnit.Assertions;
      use Crypto.Symmetric.Oneway_Blockcipher_Twofish192;
   begin
      for I in 1 .. 49 loop
	 Prepare_Key(B_Block192(Key));
	 Encrypt(PT, CT);
	 Key(16 .. 23) := Key(0 .. 7);
	 Key(0 .. 15) := Bytes(PT);
	 PT := CT;
      end loop;

      for I in CT'Range loop
	 Assert(CT(I) = Ciphertext(I), "Twofish192 Known Answer Test");
      end loop;
   end Twofish192_Oneway_Test1;
   -----------------------------------------------------------------------------

end Test.Twofish192_Oneway;
